# -*- coding: utf-8 -*-
# SmartCloudDevices
#

from smartcloud import ConnectTo, Timeout
from time import sleep
import warnings
import datetime


# Helper Functions to communicate with SmartCloud devices


def split(l, n):
    """split python list (l) after every (n) element"""
    return (l[i:i + n] for i in xrange(0, len(l), n))


def bitconv(c):
    """turn list (c) of 8bit numbers to len(c)*8 bit number"""
    r = 0
    for x in range(len(c)):
        if x == 0:  # esimest numbrit pole vaja näppida
            r = c[-1]
        else:
            r += c[-(x + 1)] << x * 8  # bitshift numbers
    return r


def date_to_date(date):
    """Make date object suitable for SmartCloud devices"""
    r = []
    year = date.year - 2000
    if not (0 <= year <= 99):
        raise ValueError("Year not in range of 2000 and 2099")
    r.append(year)  # BUG: won't work after year 2100 !
    r.append(date.month)
    r.append(date.day)
    return r


class SmartCloudDevice(object):
    """ Base class for Smart Cloud (Smart Home Bus G4) devices"""

    def __init__(self, id, subnet, scs, scr, timeout=10):
        self.id = id  # device id
        self.subnet = subnet  # devce subnet
        self.scs = scs  # SmartCloudSend obj
        self.scr = scr  # SmartCloudRecive obj
        # connection with device
        self.con = ConnectTo(self.subnet, self.id,
                            self.scs, self.scr,
                            protocol="SMARTCLOUD")
        self.timeout = timeout

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def close(self):
        """Close connection with device"""
        self.con.close()

    def is_alive(self):
        """Test if device is online or not"""
        self.con.write(0xF065)
        try:
            self.con.read()
        except Timeout:
            return False
        else:
            return True
        

    def send_datetime(self, dt=None):
        """send current date/time to the device"""
        if dt == None:  # send current time
            dt = datetime.datetime.now()
        else:  # send custom time
            dt = dt
        clock = []  # list for datetime in smartbus format 
        clock.append(dt.year - 2000)  # BUG: won't work after year 2100 !
        clock.append(dt.month)
        clock.append(dt.day)
        clock.append(dt.hour)
        clock.append(dt.minute)
        clock.append(dt.second)
        self.con.write(0x2c1, clock)


class ImpulseCounter(SmartCloudDevice):
    """class to talk with Impulse Counter"""

    def __init__(self, *largs, **kwargs):
        self.channel = int(kwargs.get("channel", 0xFF))
        super(ImpulseCounter, self).__init__(*largs, **kwargs)

    def get_nr_of_count_pkgs(self, *dates):
        """ Get Number of count packages available
        (you probably dont want to use this funtion directly)

        arguments are datetime objects
        if only one date is given then get number of packages before that date
        if two dates are given then get number of packages between thoes dates
        else get all packages"""

        # there is some weird 255 in first index of packet data

        if len(dates) == 1:
            d = date_to_date(dates[0])
            # get packages before date
            self.con.write(0xDD10, [self.channel] + d)
            pcount = self.con.read()["data"][1:]
            # turn 2 8bit numbers to 16bit number
            pcount = bitconv(pcount)

        elif len(dates) == 2:
            start_date = date_to_date(dates[0])
            end_date = date_to_date(dates[1])
            # get packgages between dates
            self.con.write(0xDD0E, [self.channel] + start_date + end_date)
            pcount = self.con.read()["data"][1:]
            pcount = bitconv(pcount)

        elif len(dates) == 0:
            d = [99, 12, 31]
            # get packages before date 2099.12.31
            self.con.write(0xDD10, [self.channel] + d)
            pcount = self.con.read()["data"][1:]
            # turn 2 8bit numbers to 16bit number
            pcount = bitconv(pcount)
        else:
            # FIXME: this is a bit wrong
            warnings.warn("Wrong number of arguments, \
                    geting all packages from Impulse counter", SyntaxWarning)
            d = [99, 12, 3]  # FIXEM: That doesn't work with current firmware !!
            # get packages before date 2099.12.31, aka everything
            self.con.write(0xDD10, [self.channel] + d)
            channel, pcount = self.con.read()["data"]
            # turn 2 8bit numbers to 16bit number
            pcount = bitconv(pcount)
        return pcount

    def get_count_package(self, package_nr):
        """ get package "package_nr"
        you need to call get_nr_of_count_pkgs() before using this funtion"""
        if package_nr > 0xFFFF:
            raise ValueError("package_nr out of range (> 0xFFFF) !")

        #counts = []
        self.con.write(0xdd12, [self.channel, package_nr / 256,
                                              package_nr % 256])
        got = self.con.read()
        #print got
        if got["data"][0] == 0xf5:
            raise Exception("Impulse counter gave a error code 0xF5")
        # Impulse counter can return multiple count packages in one packet
        # one package size is 9 bytes
        for x in split(got["data"][4:], 9):
            p = {"date": x[1:6], "count": bitconv(x[6:])}
            p["date"][0] += 2000
            p["date"] = datetime.datetime(*p["date"])
            p["date"] = p["date"] - datetime.timedelta(hours=1)
            yield p
            #counts.append(p)
        #return counts  # TODO: yield ?

    def get_counts(self, *dates):
        """Get impulse counter counts
        arguments are same as get_nr_of_count_pkgs() """
        counts = []

        #self.send_datetime()
        sleep(1)
        # first get number of packages available
        pcount = self.get_nr_of_count_pkgs(*dates)
        #print pcount
        # start to download packets
        try:
            for x in range(1, pcount + 1):
                # get packet nr x
                    for p in self.get_count_package(x):
                        counts.append(p)
        except Exception as e:
            pass
            #warnings.warn("Got ErrorCode 0xF5")
        return counts  # TODO: yield ?

    def del_counts(self, *largs, **kwargs):
        """ Delete logs from impulse counter"""
        if kwargs["all"]:
            self.con.write(0xDD18)
            got = self.con.read()["data"][0]
            if got == 0xf5:
                raise Exception("Impulse counter gave a error code 0xF5")
            return True
        else:
            raise NotImplementedError("Use all=True argument")

if __name__ == '__main__':
    print "importing smartcloud"
    from smartcloud import SmartCloudSend, SmartCloudRecive
    scs = SmartCloudSend(MyBroadcastIP="192.168.10.255")
    scr = SmartCloudRecive("192.168.10.255")
    scr.start()

    broken = []

    dev = {"id": 75, "subnet": 1}
    print "connecting"
    with ImpulseCounter(dev["id"], dev["subnet"], scs, scr) as ic:
        print "delete all"
        ic.del_counts(all=True)
        print "sending time"
        ic.send_datetime()
        print "get counts"
        #date = datetime.date(2013, 12, 31)
        date = datetime.date.today()
        #got = ic.get_nr_of_count_pkgs(date, date)  # get only todays counts
        got = ic.get_counts(date, date)
        print "got:", got
    print "ready"
    exit()
    #------------------------------------------

